/**********************************************************************************************/
/* FBL_Compressor.h                                                     					  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved                                                   					  */
/**********************************************************************************************/
/*
	FBL_Compressor -- the abstract class that specify API for compressors.
*/

#ifndef __FBL_Compressor_h 
	#define __FBL_Compressor_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#else
#	include <FBL.h>
#endif // VSDK_FRAMEWORK_STYLE


/**********************************************************************************************/
class FBL_Compressor;


/**********************************************************************************************/
class FBL_Compressor
{
	public://///////////////////////////////////////////////////////////////////////////////////

		// (note that decompression speed is relatively insensitive to the compression level)
		enum {
			kDontCompress 		= 0, 	// don't do any compression
			kFastestCompress	= 1, 	// compress as fast as possible 
			kDefaultCompress	= 6,	// default compression level
			kTightestCompress 	= 9	 	// compress as tightly as possible
		};							 	
		
		
	public://///////////////////////////////////////////////////////////////////////////////////

							FBL_Compressor( short inLevel, ulong inBufferSize = 50*1024 ) 
							{ 
								mLevel 			= inLevel;
								mBuffer 		= NULL;
								mBufferSize	= 0;
								
								MakeBuffer( inBufferSize );
							}
							
virtual						~FBL_Compressor( void ) {
								DestroyBuffer();
							}
							

virtual void 				PrepareCompress( const void* inSrc, ulong inSrcLen ) = 0;
virtual void 				FinalizeCompress( void ) = 0;

virtual void 				PrepareUncompress( const void* inDst, ulong inDstLen ) = 0;
virtual void 				FinalizeUncompress( void ) = 0;

							// Returns the maximum length of insrcLen bytes of zipped data.
virtual	ulong 				GetMaxCompressedBytes( 
									ulong 		inSrcLen ) = 0;
									
virtual ulong				GetCompressedBytes( void ) const = 0;
									
							// Compresses the data using the specified compression level. 
							// Returns TRUE if there are more data to compress.
virtual	bool 				Compress( void ) = 0;

							// Decompresses the data at inSrc. On entry inDstLen is the length of 
							// the destination buffer. On exit it's the length of the unzipped data.
virtual	void 				Uncompress( ulong inSrcLen ) = 0;
									
virtual	void				ResetBuffer( bool inCompression ) = 0;


		/* Accessories: */

		short				GetLevel( void ) const 		{ return mLevel; }
		void				SetLevel( short inLevel ) 	{ mLevel = inLevel; }
		
		void 				MakeBuffer( ulong inBufferSize )
							{
								DestroyBuffer();
								
								mBufferSize  = inBufferSize;
								mBuffer 		= new char [mBufferSize];
							}
							
		void				DestroyBuffer( void ){
								if( mBuffer )
									delete [] (char*) mBuffer;	
							}					
		
		void*				GetBuffer( void ) const 	{ return mBuffer; }
		ulong				GetBufferSize( void ) const { return mBufferSize; }
		 					

	protected:////////////////////////////////////////////////////////////////////////////////// 

		short 				mLevel;
		void*				mBuffer;
		ulong				mBufferSize;
};


/**********************************************************************************************/
class FBL_StCompressor
{
	public://///////////////////////////////////////////////////////////////////////////////////
	 
	 						FBL_StCompressor( 
	 							FBL_Compressor* inCompressor, 
	 							const void* 	inBuffer,
	 							 ulong 			inBufferLen,
	 							 bool			inCompression = true )
	 						{
	 							mCompressor = inCompressor;
	 							mCompression = inCompression;
	 						
	 							if( mCompression )
	 									mCompressor->PrepareCompress( inBuffer, inBufferLen );
	 							else	mCompressor->PrepareUncompress( inBuffer, inBufferLen );
	 						}
	 						
	 						~FBL_StCompressor( void )
	 						{
	 							if( mCompression )
	 									mCompressor->FinalizeCompress();
	 							else	mCompressor->FinalizeUncompress();
	 						}
	 						
	 						
	protected:////////////////////////////////////////////////////////////////////////////////// 	 						

		FBL_Compressor*		mCompressor;
		bool				mCompression;
};
		

/**********************************************************************************************/
#endif // __FBL_Compressor_h