/**********************************************************************************************/
/* FBL_Encryption.h                                                     					  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved                                                   					  */
/**********************************************************************************************/
/*
	FBL_Encryption -- the abstract class that specify API for compressors.
*/

#ifndef __FBL_Encryption_h 
	#define __FBL_Encryption_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#	include <VSDK/FBL_String.h>
#else
#	include <FBL.h>
#	include <FBL_String.h>
#endif // VSDK_FRAMEWORK_STYLE


/**********************************************************************************************/
class FBL_Encryption;
FBL_CLASS_DEF( FBL_Encryption );

class FBL_EncryptionKey;
FBL_CLASS_DEF( FBL_EncryptionKey );


/**********************************************************************************************/
class FBL_EncryptionKey : public FBL_Sharable
{
	public://///////////////////////////////////////////////////////////////////////////////////
	 
							FBL_EncryptionKey( const uchar* inKeyString, ulong inKeyLen )
							{
								mLen = inKeyLen;							
								mKey = new uchar[mLen];
								FBL_memcpy( mKey, inKeyString, mLen );
							}
							
							~FBL_EncryptionKey( void )
							{
								if( mKey )
									delete [] mKey;
							}
			
			ulong			Size( void ) const { return mLen; }
			uchar*			Key	( void ) const { return mKey; }	
			
							
	protected:////////////////////////////////////////////////////////////////////////////////// 
	
		uchar*				mKey;
		ulong				mLen;		 
};


/**********************************************************************************************/
class FBL_Encryption : public FBL_Sharable
{
	public://///////////////////////////////////////////////////////////////////////////////////

		enum Direction { Encrypt, Decrypt };


	public://///////////////////////////////////////////////////////////////////////////////////
				
							FBL_Encryption( FBL_EncryptionKeyCPtr /*inKey*/, Direction inDirection ) :
								mDirection( inDirection ),
								mExampleString( NULL )
							{
							}	

virtual						~FBL_Encryption( void ) 
							{
								if( mExampleString )
									delete [] mExampleString;
							}	
	
virtual	bool				IsEqual( FBL_EncryptionCPtr inOther ) const = 0;
	
		void				ProcessBlock( 
								uchar* 		inBlock, 
								ulong 		inSize );

		const char*			GetExampleString( void ) const 	{ return mExampleString; }
		ulong				GetExampleLength( void ) const 	{ return BlockSize(); }
	

	public://///////////////////////////////////////////////////////////////////////////////////
				
							// Build special string encrypted by this encryptor.
							// We can store this string and later try decrypt to see
							// if this is the same password.
		void				BuildExampleString();							

	
	protected:////////////////////////////////////////////////////////////////////////////////// 
	
							// encrypt or decrypt one block in place, pre size of inoutBlock == BlockSize()
virtual void 				DoProcessBlock( 
								uchar* 			inBlock ) const = 0;

							// encrypt or decrypt one block, may assume inBlock != outBlock.
							// the size of inBlock and outBlock == BlockSize()
virtual void 				DoProcessBlock( 
								const uchar*	inBlock, 
								uchar* 			outBlock ) const = 0;
	
							// block size of the cipher in bytes 
virtual 					ulong BlockSize( void ) const = 0;


	protected:////////////////////////////////////////////////////////////////////////////////// 
	
		Direction			mDirection;
		char*				mExampleString;

};


/**********************************************************************************************/
// Factory of FBL_Encryption class implementation.

enum FBL_EncryptionTypes
{
	kBlowFish	= 1
};


FBL_Encryption* FBL_CreateEncryptor( 
	FBL_EncryptionTypes 		inType, 
	FBL_EncryptionKeyCPtr 		inKey, 
	FBL_Encryption::Direction	inDirection );


/**********************************************************************************************/
#endif // __FBL_Compressor_h


