/**********************************************************************************************/
/* FBL_String.h                                                        						  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                       					  */
/* All Rights Reserved                                                   					  */
/**********************************************************************************************/

#ifndef __FBL_String_h 
	#define __FBL_String_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#else
#	include <FBL.h>
#endif // VSDK_FRAMEWORK_STYLE

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class FBL_EXP FBL_String 
{
    public:////////////////////////////////////////////////////////////////////////////////////

static FBL_String		sEmpty;	// to be able return reference on FBL_String


    public:////////////////////////////////////////////////////////////////////////////////////

		               	FBL_String( void );
		               	FBL_String( ulong inLength );			// just allocate buffer
		               	
		               	FBL_String( ulong inLength, char ch );
						FBL_String( const char* str );
               			FBL_String( const char* str, ulong inLength ); // use part of str, -1 if all.

		               	FBL_String( const FBL_String& Src );

						FBL_String( const uchar* inStr ); // build C string from the pascal string.

virtual	 				~FBL_String();

        ///////////////////////////////////////////////////////////////////////////////////////

        operator		const char* () const	{ return mString; }
        operator		char* ()				{ return mString; }

        char*			begin( void ) 			{ return mString; }
const 	char*			begin( void ) const 	{ return mString; }
                
        char*			end( void ) 			{ return mString + mLen; }
const 	char*			end( void ) const 		{ return mString + mLen; }
                
      	ulong			GetLength( void ) const				{ return mLen; 		}
        ulong 			GetAllocatedLength( void ) const 	{ return mAllocLen; }
        
        bool			IsEmpty( void ) const 	{ return bool(mLen == 0); }
        void			Clear( void );

						// ZERO based index
        char&			operator()( ulong inIndex ) const { return mString[inIndex]; }

         				/** This tricky function can be used to avoid coping of buffer */
        char*			GetBufferOwnership( void ) { 
        					char* p = mString; 
        					Init();
        					return p;
        				};	

       // Overloaded assignment:
        
		FBL_String&		operator= ( const FBL_String& Src );
		FBL_String&		operator= ( char ch );
		FBL_String&		operator= ( const char* str );
		FBL_String&		operator= ( const uchar* str );	// pascal string

  
  		// Assigment/appending of numbers to FBL_String:

		FBL_String&		SetInt   	( long 		Value );
		FBL_String&		SetUInt  	( ulong  	Value );
		FBL_String&		SetLLong	( llong 	Value );
		FBL_String&		SetULLong 	( ullong  	Value );
		FBL_String&		SetDouble	( double 	Value );
//		FBL_String&		SetLDouble	( ldouble 	Value );

		FBL_String&		AddInt    	( long   	Value );
		FBL_String&		AddUInt   	( ulong  	Value );
		FBL_String&		AddLLong  	( llong   	Value );
		FBL_String&		AddULLong 	( ullong  	Value );
		FBL_String&		AddDouble 	( double 	Value );
//		FBL_String&		AddLDouble	( ldouble 	Value );

		FBL_String&		operator<<  ( long   	Value )	{ return AddInt(Value); 	}
		FBL_String&		operator<<  ( ulong  	Value )	{ return AddUInt(Value); 	}
		FBL_String&		operator<<  ( llong   	Value )	{ return AddLLong(Value); 	}
		FBL_String&		operator<< 	( ullong  	Value )	{ return AddULLong(Value); 	}
		FBL_String&		operator<< 	( double 	Value )	{ return AddDouble(Value); 	}
//		FBL_String&		operator<<	( ldouble	Value )	{ return AddLDouble(Value); }

		
		// Exctraction of numbers from FBL_String:
		
						operator uchar( void ) const;
						operator char ( void ) const;
						operator ushort( void ) const;
						operator short ( void ) const;
						operator long  ( void ) const;
						operator ulong ( void ) const;
						operator llong ( void ) const;	
						operator ullong( void ) const;	
						operator float ( void ) const;
						operator double ( void ) const;
//						operator ldouble ( void ) const;


        // FBL_String concatenation:
        
		FBL_String&		operator+= ( const FBL_String& str );
		FBL_String&		operator+= ( char ch );
		FBL_String&		operator+= ( const char* str );

		FBL_String&		operator<< ( const FBL_String& str )	{ return operator+=(str);	}
		FBL_String&		operator<< ( char ch )					{ return operator+=(ch);	}
		FBL_String&		operator<< ( const char* str )			{ return operator+=(str);	}


FBL_EXP	friend FBL_String operator+ ( const FBL_String& string1, const FBL_String& 	string2);
FBL_EXP friend FBL_String operator+ ( const FBL_String& string ,       char    		ch     );
FBL_EXP friend FBL_String operator+ (       char    	ch     , const FBL_String& 	string );
FBL_EXP friend FBL_String operator+ ( const FBL_String& string , const char*   		str    );
FBL_EXP friend FBL_String operator+ ( const char*   	str    , const FBL_String& 	string );


        // FBL_String comparison:
        
        void			CaseSensetive( bool sens ) 	{ sCaseSensetive = sens; }
        bool			CaseSensetive( void ) const { return sCaseSensetive; }
 
						// compare according to sCaseSensetive flag		
		short			Compare(const char* str) const; 

        short			CompareNoCase(const char* str, ulong inHowMuch = (ulong) -1 ) const;
		static short	CompareNoCase(const char* s1, const char* s2, ulong inHowMuch = (ulong) -1 ); 
        
		// Simple sub-string extraction:
        
		FBL_String&		Insert( char* inStr, ulong inPos );

		FBL_String		Mid  ( ulong nFirst, ulong nCount ) const;
		FBL_String		Mid  ( ulong nFirst ) const;
		FBL_String		Left ( ulong nCount ) const;
		FBL_String		Right( ulong nCount ) const;

		FBL_String&		Delete( ulong First, ulong Count );
		FBL_String&		Truncate( ulong NewLen );
        				
		FBL_String		SpanIncluding( const char* charSet ) const;
		FBL_String		SpanExcluding( const char* charSet ) const;

		FBL_String&		MakeReverse( void );

		FBL_String&		MakeUpper( void );
		FBL_String&		MakeLower( void );

        // Searching (return starting index, or -1 if not found)
        // look for a single character match

		long			Find( char ch, ulong InStart = 0 ) const;			// like "C" strchr
		long			ReverseFind( char ch ) const;
		long			FindOneOf( const char* charSet, ulong InStart = 0 ) const;

        // Look for a specific sub-string:
        
		long			Find( const char* str, ulong InStart = 0 ) const;	// like "C" strstr


   public://///////////////////////////////////////////////////////////////////////////////

						// Convert inStr and assign to self. 
						// If inStr is NULL then convert self.
		FBL_String&		Latin1_UTF8( const char* inStr = NULL, ulong inLength = 0 );
		FBL_String&		UTF8_Latin1( const char* inStr = NULL, ulong inLength = 0 );

						// Encode for XML chars as '&' '<' '>'		
		FBL_String&		AddXmlEncoding( void );
		FBL_String&		RemoveXmlEncoding( void );


   public://///////////////////////////////////////////////////////////////////////////////

static 	char*			strupr(char *str)
						{
							char* p = str;
							while( *p ){
								*p = (char) toupper(*p);					
								p++;
							}
							return str;
						}

static char* 			strlwr(char *str)
						{
							char* p = str;
							while( *p ){
								*p = (char) tolower(*p);
								p++;
							}
							return str;
						}


   public://///////////////////////////////////////////////////////////////////////////////

		void    		Init( void );
		void    		CopyObject( const FBL_String& Src );

		void    		AllocBuffer( ulong inLen, bool inInit = true );

		FBL_String& 	AllocCopy( FBL_String& dest, ulong nCopyLen,
                           		   ulong nCopyIndex, ulong nExtraLen ) const;

		FBL_String& 	AssignCopy ( const char* SrcData, ulong nSrcLen );

		FBL_String& 	ConcatCopy( const char* Src1Data, ulong nSrc1Len,
                            		const char* Src2Data, ulong nSrc2Len  );

		FBL_String&		ConcatInPlace( const char* SrcData, ulong nSrcLen );

		void			SetLength( ulong inLength ){ 
							if( inLength > mAllocLen )
								inLength = mAllocLen;
								
							mString[ mLen = inLength ] = 0; 
						}


    protected://///////////////////////////////////////////////////////////////////////////////

static	void 			SafeDelete( char* pch ){
						    if( pch ) delete [] pch;
						}

static	ulong 			SafeStrlen( const char* str ){
						    return (str) ? ulong(strlen(str)) : ulong(0);
						}
						

    protected://///////////////////////////////////////////////////////////////////////////////

        // Note: an extra character is always allocated
        
        char*         	mString;
        ulong        	mAllocLen; 			// does not include terminating 0
        ulong        	mLen;

static  bool         	sCaseSensetive;

};


/**********************************************************************************************/
inline FBL_String& FBL_String::operator= ( const FBL_String& Src )
{
	AssignCopy( Src.mString, Src.mLen );
	return *this;
}

/**********************************************************************************************/
inline FBL_String& FBL_String::operator= ( const char* str )
{
	AssignCopy( str, SafeStrlen( str ) );
	return *this;
}

/**********************************************************************************************/
inline FBL_String& FBL_String::operator= ( char ch )
{
    AssignCopy( &ch, 1 );
    return *this;
}

/**********************************************************************************************/
inline FBL_String& FBL_String::operator = ( const uchar* inStr )
{
	AssignCopy( (char*)(inStr + 1), (ulong) inStr[0] );
	return *this;

}

/**********************************************************************************************/
inline FBL_String& FBL_String::operator += ( const char* str )
{
    ConcatInPlace( str, SafeStrlen(str) );
    return *this;
}

/**********************************************************************************************/
inline FBL_String& FBL_String::operator += ( char ch )
{
    ConcatInPlace( &ch, 1 );
    return *this;
}

/**********************************************************************************************/
inline FBL_String& FBL_String::operator += ( const FBL_String& string )
{
    ConcatInPlace( string.mString, string.mLen );
    return *this;
}


#pragma mark -


/**********************************************************************************************/
inline short FBL_String::CompareNoCase(const char* str, ulong inHowMuch ) const 
{ 
	return CompareNoCase( mString, str, inHowMuch );
}

/**********************************************************************************************/
inline FBL_String FBL_String::Mid( ulong First ) const
{
    return Mid( First, mLen - First );
}


#pragma mark -


/**********************************************************************************************/
// must be used to get first letter as char
inline FBL_String::operator char( void ) const 
{
	return (char) *mString;
}

/**********************************************************************************************/
// must be used to convert numeric String into BYTE number.
inline FBL_String::operator uchar( void ) const 
{
	return (uchar) atol( mString );	
}

/**********************************************************************************************/
inline FBL_String::operator short( void ) const 
{
	return (short) atol( mString );
}

/**********************************************************************************************/
inline FBL_String::operator ushort( void ) const 
{
	char* end;
	return (ushort) strtoul( mString, &end, 10 );				
}

/**********************************************************************************************/
inline FBL_String::operator long( void ) const 
{
	return atol( mString );
}

/**********************************************************************************************/
inline FBL_String::operator ulong( void ) const 
{
	char* end;
	return strtoul( mString, &end, 10 );				
}

/**********************************************************************************************/
inline FBL_String::operator llong( void ) const 
{
	llong z = 0;

	sscanf( mString, "%lli", &z );
	return z;
}

/**********************************************************************************************/
inline FBL_String::operator ullong( void ) const 
{
	ullong z = 0;

	sscanf( mString, "%llu", &z );
	return z;
}

/**********************************************************************************************/
inline FBL_String::operator float( void ) const 
{
	return (float) atof( mString );
}

/**********************************************************************************************/
inline FBL_String::operator double( void ) const 
{
	return atof( mString );
}

/**********************************************************************************************/
/*inline FBL_String::operator ldouble( void ) const 
{
	ldouble z = 0;

	sscanf( mString, "%Lg", &z );
	return z;
}*/

#pragma mark -


/**********************************************************************************************/
inline FBL_String operator+( const FBL_String& string1, const FBL_String& string2 )
{
    FBL_String s;
    return s.ConcatCopy( string1.mString, string1.mLen,
                         string2.mString, string2.mLen  );
}

/**********************************************************************************************/
inline FBL_String operator+( const FBL_String& string, const char* str )
{
    FBL_String s;
    return s.ConcatCopy( string.mString	, string.mLen,
                         str            , FBL_String::SafeStrlen(str) );
}

/**********************************************************************************************/
inline FBL_String operator+( const char* str, const FBL_String& string )
{
    FBL_String s;
    return s.ConcatCopy( str           , FBL_String::SafeStrlen(str),
                         string.mString, string.mLen   );
}

/**********************************************************************************************/
inline FBL_String operator+( const FBL_String& string1, char ch )
{
    FBL_String s;
    return s.ConcatCopy( string1.mString, string1.mLen, &ch, 1 );
}

/**********************************************************************************************/
inline FBL_String operator+( char ch, const FBL_String& string )
{
    FBL_String s;
    return s.ConcatCopy( &ch, 1, string.mString, string.mLen );
}

/**********************************************************************************************/
inline bool operator == ( const FBL_String& s1, const FBL_String& s2 )
{
    return bool( s1.Compare(s2) == 0 );
}

/**********************************************************************************************/
inline bool operator == ( const FBL_String& s1, const char*  s2 )
{
    return bool( s1.Compare(s2) == 0 );
}

/**********************************************************************************************/
inline bool operator == ( const char* s1, const FBL_String& s2 )
{
    return bool( s2.Compare(s1) == 0 );
}

/**********************************************************************************************/
inline bool operator != ( const FBL_String& s1, const FBL_String& s2 )
{
    return bool( s1.Compare(s2) != 0 );
}

/**********************************************************************************************/
inline bool operator != ( const FBL_String& s1, const char* s2 )
{
    return bool( s1.Compare(s2) != 0 );
}

/**********************************************************************************************/
inline bool operator != ( const char* s1, const FBL_String& s2 )
{
    return bool( s2.Compare(s1) != 0 );
}

/**********************************************************************************************/
inline bool operator < ( const FBL_String& s1, const FBL_String& s2 )
{
	return bool( s1.Compare(s2) < 0 );
}
/**********************************************************************************************/
inline bool operator < ( const FBL_String& s1, const char* s2 )
{
    return bool( s1.Compare(s2) < 0 );
}

/**********************************************************************************************/
inline bool operator < ( const char* s1, const FBL_String& s2 )
{
    return bool( s2.Compare(s1) >= 0 );
}

/**********************************************************************************************/
inline bool operator <= ( const FBL_String& s1, const FBL_String& s2 )
{
    return bool( s1.Compare(s2) <= 0 );
}

/**********************************************************************************************/
inline bool operator <= ( const FBL_String& s1, const char* s2 )
{
    return bool( s1.Compare(s2) <= 0 );
}

/**********************************************************************************************/
inline bool operator <= ( const char* s1, const FBL_String& s2 )
{
    return bool( s2.Compare(s1) > 0 );
}

/**********************************************************************************************/
inline bool operator > ( const FBL_String& s1, const FBL_String& s2 )
{
    return bool( s1.Compare(s2) > 0 );
}

/**********************************************************************************************/
inline bool operator > ( const FBL_String& s1, const char* s2 )
{
    return bool( s1.Compare(s2) > 0 );
}

/**********************************************************************************************/
inline bool operator > ( const char* s1, const FBL_String& s2 )
{
    return bool( s2.Compare(s1) <= 0 );
}

/**********************************************************************************************/
inline bool operator >= ( const FBL_String& s1, const FBL_String& s2 )
{
    return bool( s1.Compare(s2) >= 0 );
}

/**********************************************************************************************/
inline bool operator >= ( const FBL_String& s1, const char* s2 )
{
    return bool( s1.Compare(s2) >= 0 );
}

/**********************************************************************************************/
inline bool operator >= ( const char* s1, const FBL_String& s2 )
{
    return bool( s2.Compare(s1) < 0 );
}

/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif	// __String_h
